/*****************************************************************************/
/* N.A.D.A.R. - A network tank battle game.                                  */
/*                                                                           */
/* N.A.D.A.R. Copyright (c) 2000-2001 Sakai Hiroaki.                         */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* N.A.D.A.R. is a network tank battle game. You get into a tank,            */
/* operate it, and destroy other tanks to survive!                           */
/*****************************************************************************/

/*****************************************************************************/
/* ФΥᥤ                                                            */
/*****************************************************************************/

#include <signal.h>

#include "Game.h"
#include "arguments.h"

Game game = NULL;

static void quit(int value)
{
  ObjListData current;
  Stage stage;
  Player player;

  if (game) {
    stage = GameGetStage(game);
    if (stage) {
      for (current = ObjList_GetStart(StageGetPlayerList(stage));
	   !ObjList_IsEndEdge(StageGetPlayerList(stage), current);
	   current = ObjListData_GetNext(current)) {
	player = (Player)ObjListData_GetObj(current);
	if (PlayerGetState(player) != PLAYER_ACTIVE) continue;
	PathSendNoneData(PlayerGetPath(player), DELETE_YOU);
	PathFlushSendBuffer(PlayerGetPath(player));
      }
    }
    game = GameDestroy(game);
  }

  fprintf(stderr, "Terminated.\n");

  exit (0);
}

static void show_help()
{
  printf("\n");
  printf("                                 N.A.D.A.R.\n");
  printf("                        A network tank battle game.\n");
  printf("\n");
  printf("             N.A.D.A.R. Copyright (c) 2000-2001 Sakai Hiroaki.\n");
  printf("                            All Rights Reserved.\n");
  printf("\n");
  printf("This is ");
  printf(VERSION);
  printf(" server. Be careful that different versions of N.A.D.A.R. are unconnectable.\n");
  printf("\n");
  printf("N.A.D.A.R. is a client-server model program. ");
  printf("This is N.A.D.A.R.\'s server program.\n");
  printf("\n");
  printf("Starting N.A.D.A.R. server.\n");
  printf("If you want to play N.A.D.A.R., Start N.A.D.A.R. clients as\n");
  printf("\n");
  printf("  %% nadar -s [Server\'s hostname] -p [Player\'s name]\n");
  printf("\n");
  printf("If playing speed is very slow, ");
  printf("run clients with size option as below.\n");
  printf("\n");
  printf("  %% nadar -s [Server\'s hostname] -p [Player\'s name] -size 30\n");
  printf("\n");
  printf("If you try the peephole mode, run server with peephole option as below.\n");
  printf("\n");
  printf("  %% nadars -peephole\n");
  printf("\n");
  printf("If you try the non-backward mode, run server with no-backward option\n");
  printf("as below.\n");
  printf("\n");
  printf("  %% nadars -no-backward\n");
  printf("\n");
}

static void show_options()
{
  printf("Options :\n\n");
  printf("\t-h, -help : Show help messages.\n");
  printf("\t-host, -hostname [hostname] : The hostname of the server.\n");
  printf("\t-silent : Enter silent mode.\n");
  printf("\t-no-silent : Enter non-silent mode. (Default)\n");
  printf("\t-port : The port number of socket. Default is 12010.\n");
  printf("\t-width [width], -height [height] : The screen size on peephole mode.\n");
  printf("\t-map-width [width], -map-height [height] : The map size.\n");
  printf("\t-peep, -peephole : Enter peephole game mode.\n");
  printf("\t-no-peep, -no-peephole : Enter non-peephole game mode. (Default)\n");
  printf("\t-backward : Tank can go backward. (Default)\n");
  printf("\t-no-backward : Tank cannot go backward.\n");
  printf("\t-interval [milli seconds] : Game cycle time. (milli seconds)\n");
  printf("\t-timeout [milli seconds] : Timeout in playing game. (milli seconds)\n");
  printf("\t-max-players [number] : Maximum number of players.\n");
  printf("\n");
}

static void help()
{
  show_help();
  show_options();
  exit (0);
}

static char * server_name = NULL;
static int port           = PORT;
static int silent         = 0;
static int width          = 0;
static int height         = 0;
static int map_width      = 0;
static int map_height     = 0;
static int peeping        = DEFAULT_PEEPHOLE_FLAG;
static int backward       = DEFAULT_BACKWARD_FLAG;
static int interval_time  = DEFAULT_INTERVAL_TIME;
static int timeout_time   = DEFAULT_TIMEOUT_TIME;
static int max_players    = DEFAULT_MAX_PLAYERS;

static Argument arguments[] = {
  {"-h",                ARGUMENTS_FUNCTION, (void *)(&help)},
  {"-help",             ARGUMENTS_FUNCTION, (void *)(&help)},
  {"-host",             ARGUMENTS_STRING,   (void *)(&server_name)},
  {"-hostname",         ARGUMENTS_STRING,   (void *)(&server_name)},
  {"-port",             ARGUMENTS_INTEGER,  (void *)(&port)},
  {"-silent",           ARGUMENTS_FLAG_ON,  (void *)(&silent)},
  {"-no-silent",        ARGUMENTS_FLAG_OFF, (void *)(&silent)},
  {"-width",            ARGUMENTS_INTEGER,  (void *)(&width)},
  {"-height",           ARGUMENTS_INTEGER,  (void *)(&height)},
  {"-map-width",        ARGUMENTS_INTEGER,  (void *)(&map_width)},
  {"-map-height",       ARGUMENTS_INTEGER,  (void *)(&map_height)},
  {"-peep",             ARGUMENTS_FLAG_ON,  (void *)(&peeping)},
  {"-peephole",         ARGUMENTS_FLAG_ON,  (void *)(&peeping)},
  {"-no-peep",          ARGUMENTS_FLAG_OFF, (void *)(&peeping)},
  {"-no-peephole",      ARGUMENTS_FLAG_OFF, (void *)(&peeping)},
  {"-backward",         ARGUMENTS_FLAG_ON,  (void *)(&backward)},
  {"-no-backward",      ARGUMENTS_FLAG_OFF, (void *)(&backward)},
  {"-interval",         ARGUMENTS_INTEGER,  (void *)(&interval_time)},
  {"-timeout",          ARGUMENTS_INTEGER,  (void *)(&timeout_time)},
  {"-max-players",      ARGUMENTS_INTEGER,  (void *)(&max_players)},
  {NULL,                ARGUMENTS_NONE,     NULL}
};

int main(int argc, char * argv[])
{
  int i;

  signal(SIGINT , quit);
  signal(SIGTERM, quit);

  InitializeRandom();

  Arguments_Read(&argc, argv, arguments);

  if (port < 1000) port = PORT;

  if (!silent) show_help();

  if (!peeping) {
    if ((map_width  < 10) || (map_width  > 160)) map_width  = MAP_WIDTH;
    if ((map_height < 10) || (map_height > 160)) map_height = MAP_HEIGHT;
    width  = map_width;
    height = map_height;
  } else {
    if ((map_width  < 10) || (map_width  > 160)) map_width  = PEEP_MAP_WIDTH;
    if ((map_height < 10) || (map_height > 160)) map_height = PEEP_MAP_HEIGHT;
    if ((width  < 3) || (width  > 50)) width  = PEEP_WIDTH;
    if ((height < 3) || (height > 50)) height = PEEP_HEIGHT;
  }

  if ((interval_time < 0) || (interval_time > 10000)) interval_time = 0;
  if (timeout_time < 0) timeout_time = 0;
  if (max_players <  1) max_players = 1;
  if (max_players > 64) max_players = 64;

  game = GameCreate(server_name, port,
		    map_width, map_height, width, height, peeping, backward,
		    interval_time, timeout_time, max_players);
  if (game == NULL) exit (1);

  for (i = 1; i < argc; i++) {
    fprintf(stderr, "Unknown option : %s\n", argv[i]);
  }

  GameStart(game, silent);
  game = GameDestroy(game);

  exit (0);
}

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
